package cn.com.qmth.examcloud.core.basic.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import cn.com.qmth.examcloud.core.basic.dao.CourseRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseSpeciatlyRelationEntity;

@Service
public class CourseServiceImpl {

	@Autowired
	CourseRepo courseRepo;

	private Specification<CourseEntity> getSpecification(CourseEntity courseCriteria, Long specialtyId) {
		Specification<CourseEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			if (!StringUtils.isEmpty(courseCriteria.getRootOrgId())) {
				predicates.add(cb.equal(root.get("orgId"), courseCriteria.getRootOrgId()));
			}
			if (!StringUtils.isEmpty(courseCriteria.getName())) {
				predicates.add(cb.like(root.get("name"), "%" + courseCriteria.getName() + "%"));
			}
			if (!StringUtils.isEmpty(courseCriteria.getCode())) {
				predicates.add(cb.like(root.get("code"), "%" + courseCriteria.getCode() + "%"));
			}
			if (!StringUtils.isEmpty(courseCriteria.getLevel())) {
				predicates.add(cb.equal(root.get("level"), courseCriteria.getLevel()));
			}
			if (!StringUtils.isEmpty(courseCriteria.getEnable())) {
				predicates.add(cb.equal(root.get("enable"), courseCriteria.getEnable()));
			}
			if (specialtyId != null) {
				Subquery<CourseSpeciatlyRelationEntity> relationshipSubquery = query
						.subquery(CourseSpeciatlyRelationEntity.class);
				Root<CourseSpeciatlyRelationEntity> residencyRelationshipSubqueryRoot = relationshipSubquery
						.from(CourseSpeciatlyRelationEntity.class);
				relationshipSubquery.select(residencyRelationshipSubqueryRoot);
				Predicate predicate = cb.equal(residencyRelationshipSubqueryRoot.get("specialtyId"),
						specialtyId);
				Predicate predicate2 = cb.equal(residencyRelationshipSubqueryRoot.get("courseId"),
						root.get("id"));
				relationshipSubquery.where(cb.and(predicate, predicate2));
				predicates.add(cb.exists(relationshipSubquery));
			}
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		return specification;
	}

	public List<CourseEntity> findAll(CourseEntity courseCriteria) {
		Specification<CourseEntity> specification = getSpecification(courseCriteria, null);
		return courseRepo.findAll(specification);
	}

	public void setCourseEnable(List<Long> courseIds, boolean enable) {
		for (Long courseId : courseIds) {
			CourseEntity course = courseRepo.findOne(courseId);
			course.setEnable(enable);
			course.setUpdateTime(new Date());
			courseRepo.saveAndFlush(course);
		}
	}

	@Transactional
	public void delete(String ids) {
		List<Long> courseIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long courseId : courseIds) {
			courseRepo.delete(courseId);
		}
	}

}
